// compressfile.cpp: implementation of the CCompressFile class.
//
//////////////////////////////////////////////////////////////////////


#include "compressfile.h"

#include "PackFile.h"
#include "DataManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

namespace FileSystem
{

CResult CCompressFile::initialize(CDataManager* pManager,
								  TPacksListElement* packFile, 
								  TFileInfoElement* element)
{

	//for future format			
	char stringBuffer[1024];

	//init vars
	buffer = NULL;
	readBuffer = NULL;
	currentPos = 0;
	totalSize = 0;
	numBlocks = 0;
	blockOffsets = NULL;	
	file = NULL;
	lastBlockRead = -1;

	//call base
	CResult result = Inherit::initialize(pManager);

	if (result == resultOk)	
	{	
		//stotre total size,blocksize,offset of the file
		totalSize = element->info.uncompressSize;
		blockSize = packFile->pack.getHeader()->blocksize;	
		offset = element->info.offset;

		//make the buffers
		buffer = new unsigned char[blockSize];	
		readBuffer = new unsigned char[blockSize];	

		//calculate how many blocks we have and make an array of block offsets
		numBlocks = (totalSize/blockSize)+1;
		blockOffsets = new unsigned long[numBlocks+1];						
		
		//if we haven't memory
		if( (buffer==NULL) || (readBuffer==NULL))
		{
			result = CResult(codeFail,0,"CCompressFile::initialize - Couldn't get memory");
		}
		else
		{
			//get the file from the pack
			char sFile[MAX_PATH];
			packFile->pack.getFileName(sFile);

			file = fopen(sFile,"rb");

			if ( file == NULL)
			{

				//error string
				sprintf(stringBuffer,
					"CCompressFile::initialize - couldn't open file %s",sFile);

				result = CResult(codeFail,0,stringBuffer);
			}
			else
			{	
				//seek to the block aree of the file
				fseek(file,offset-1,SEEK_SET);	

				//init vars
				unsigned long int readedSize;
				unsigned long int compressSize;
				unsigned long int currentOffset = 0;

				unsigned long int blockCount;

				//fill the array of block offsets
				for(blockCount=0;blockCount<numBlocks;blockCount++)
				{
					blockOffsets[blockCount] = currentOffset;
					

					fread(&readedSize,1,sizeof(readedSize),file);
					fread(&compressSize,1,sizeof(compressSize),file);

					fseek(file,compressSize,SEEK_CUR);
					currentOffset+=compressSize;
				}
	
			}
		}
	}

	if (result!= resultOk)
	{
		totalSize = 0;
	}

	return result;
}


int CCompressFile::eof()
{
	//if we are at the end
	return ( currentPos>=totalSize)?TRUE:FALSE;
}


int CCompressFile::read(long amount,char* obuffer)
{
	int read = 0;

	//starting and end byte
	unsigned long int readFromByte = currentPos;
	unsigned long int readEndByte = currentPos+amount;

	//dont pass end
	if (readEndByte>totalSize)
		readEndByte = totalSize;

	//starting and end block
	unsigned long int readFromBlock= (currentPos/blockSize);
	unsigned long int readToBlock= (readEndByte/blockSize);

	//in the first block how many byte we are father from begining
	unsigned long int starBlockOffset = (readFromByte)-(readFromBlock*blockSize);	


	//read the blocks
	char* sDest = obuffer;
	char* sOrigin = NULL;
	unsigned long int amountBlock = blockSize;
	for (unsigned long blockCounter=readFromBlock;blockCounter<(readToBlock+1);blockCounter++)
	{
		//origin the begining of the block
		sOrigin = (char*)buffer;

		if (blockCounter==readFromBlock)
		{
			//go to the correct place
			sOrigin+=starBlockOffset;	
		}

		if (readFromBlock==readToBlock)
		{
			//on read pass
			amountBlock = (readEndByte-readFromByte);
		}
		else
		{
			//just the part we have left
			amountBlock = read - (readEndByte-readFromByte);
		}

		//we read the block
		if (readBlock(blockCounter)==resultOk)
		{
			//copy the part
			memcpy(sDest,sOrigin,amountBlock-1);
			sDest+=amountBlock;
			read+=amountBlock;
		}
		else
		{
			//error reading or uncomrpessing
			read = -1;
			blockCounter = readToBlock;
		}
	}


	if ( read!=-1)
	{	
		//update internal pos
		currentPos+=read;

		if(currentPos>=totalSize)
			currentPos=totalSize;
	}

	return read;
}

int CCompressFile::seek(long amount,int pos)
{
	//just update internal pos, maybe need some cheking
	switch(pos)
	{
		case SEEK_CUR:
		//Current position of file pointer
			currentPos+=amount;
		break;

		case SEEK_END:
		//End of file
			currentPos=totalSize-amount;
		break;

		case SEEK_SET:
		//Beginning of file
			currentPos=amount;
		break;
	}
	
	return 0;
}

CResult CCompressFile::finalize()
{
	return Inherit::finalize();

	//free memory and close file

	if(buffer!=NULL)
	{
		delete [] buffer;
	}

	
	if(readBuffer!=NULL)
	{
		delete [] readBuffer;
	}

	if(file!=NULL)
	{
		fclose(file);
	}

	if (blockOffsets!=NULL)
	{
		delete [] blockOffsets;
	}
}

CResult CCompressFile::readBlock(int num)
{	
	
	CResult result = resultOk;

	//if we have this block in memory, skip
	if ( lastBlockRead != num)
	{
		//go to the star of the block
		fseek(file,offset-1,SEEK_SET);
		fseek(file,blockOffsets[num],SEEK_CUR);

		//zeros to the buffers
		ZeroMemory(readBuffer,blockSize);
		ZeroMemory(buffer,blockSize);

		unsigned long int readedSize;
		unsigned long int compressSize;

		//read the compres/uncompres size of the data
		fread(&readedSize,1,sizeof(readedSize),file);
		fread(&compressSize,1,sizeof(compressSize),file);
			

		//read the amount of data that its compresed
		fread(readBuffer,1,compressSize,file);

		
		//uncompress the data into the out buffer
		result = m_pManager->uncompressBlock(buffer,&readedSize,readBuffer,
				compressSize);	

		if (result == resultOk)
		{
			//store this block as read
			lastBlockRead = num;	
		}
	}

	return result;
}

char* CCompressFile::gets(char* buffer,int maxLen)
{
	//just get the bytes
	char* out = NULL;
	int readed = read(1024,buffer);

	if (readed!=-1)
	{
		//try to find and end char
		char* end = strstr(buffer,"\n");
		if (end!=NULL)
		{	
			//set the next to the end as end of string, if he isn't the end itself
			if (end!=&buffer[maxLen])
			{
				*(end+1)='\0';
			}
		}

		out = buffer;
	}
	return out;
}

}

