// Result.h: interface for the CResult class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(RESULT__INCLUDED_)
#define RESULT__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>
namespace FileSystem
{

//!  Common enums for erros codes. 
/*!
  This enums values are common error codes  
		\ingroup Base
		\sa CResult
*/
enum TResultsCode
{
	//! Ok result. No errors.
	codeOk	= 0x0000,
	//! Fail result. Some errors may hapend.
	codeFail= 0x0001

};

//!  Class for encapsulate error handling. 
/*!
  This class give suport for error controls.
  A CResult object it's has three importan fields.

	- \a code : a common TResultsCode.
	- \a subcode : an arbitrary subcode value.
	- \a string : a text describin the error.

  This object it's used as result in many 
  functions. After a function call you
  could check this object to see if a error
  has happen.

		\ingroup Base
		\sa TResultsCode
*/

class CResult;

class CResult
{
public:

	// Constructor for object without info.
	/*!	This constructor it's usually used when
		you whant to just make a CResult object
		for getting a result from a function.
	*/
	CResult();

	// Constructor with init info.
	/*!	This constructor it use when you whant
	to make your a CResul to return custom errors.
	*/	
	CResult(TResultsCode newCode,WORD newSubcode,char* newString);

	//!  Default destructor. 
	/*!
		Called before the object destruction.
	*/
	virtual ~CResult();

	//! Actual result code
	TResultsCode code;

	//! custom subcode
	WORD subcode;

	//! description of the result
	char string[4024];

	//! copy operator
	/*! copy an CResult to other.
	*/
	CResult &operator = (CResult origin);

};

//! Operator == 
/*!
    Compare a par of CResult using the code param
*/
bool operator==(CResult& result1, CResult& result2);

//! Operator != 
/*!
    Compare a par of CResult using the code param
*/
bool operator!=(CResult& result1, CResult& result2);

//! Ok Result
/*!
    Standar CResult indicates ok result
*/
extern CResult resultOk;

//! Fail Result
/*!
    Standar CResult indicates fail result
*/
extern CResult resultFail;

}
#endif // !defined(RESULT__INCLUDED_)
